/***************************************************************************
   ktt.h  -  KTTApp

   begin     : 2002.11.08
   copyright : (C) 2002 by MiMaS
   email     : mimas@mimas.ceti.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KTT_H
#define KTT_H
 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for KDE 
#include <kapp.h>
#include <klocale.h>
#include <kmainwindow.h>
#include <kaccel.h>
#include <kaction.h>
#include <kurl.h>

// application specific includes
#include "resources.h"
#include "narmacil/transcription.h"

// forward declaration of the KTT classes
class KTTDoc;
class KTTView;
class CTranscription;

/** KTT main configuration structure */
typedef struct KTTConfig
{
  bool autoTranscribe;
  bool clearTarget;
  int onStartup;
  QString recentMode;
  QString startupMode;
  QString currentMode;
  QStringList tengwarFonts;
  bool fullPathCaption;
//  bool saveOnExit;
  struct {
    bool alert;
    bool smart;
    bool digits;
    bool LSD;
    bool decimal;
    bool separateNums;    
  } narmacil;
};

class KTTApp : public KMainWindow
{
  Q_OBJECT

  friend class KTTView;
  friend class KTTSetupDlg;

  public:
    KTTApp(QWidget* parent=0, const char* name=0);
    ~KTTApp();

    void openDocumentFile(KURL& url, const int type);
    KTTDoc *getDocument() const;

  protected:
    void saveOptions();
    void readOptions();

    void initActions();
    void initWhatsThis();
    void initStatusBar();
    void initDocumentView();
    void setDualCaption();

    void fileOpen(const int type);
    void fileSaveAs(const int type);
    void loadNarmacilMode(const KURL& url);

    virtual bool queryClose();
    virtual bool queryExit();
    virtual void saveProperties(KConfig *_cfg);
    virtual void readProperties(KConfig *_cfg);

  public slots:
    void slotFileNew();
    void slotFileOpenRoman();
    void slotFileOpenTengwar();
    void slotFileOpenRecent(const KURL& _url);
    void slotFileSave();
    void slotFileSaveAs();
    void slotFileSaveAsRoman();
    void slotFileSaveAsTengwar();    
    void slotFileClose();
    void slotFilePrint();
    void slotFileQuit();

    void slotSaveAvailable();
    void slotCopyAvailable(bool yes);
    void slotEditCut();
    void slotEditCopy();
    void slotEditPaste();

    void slotViewToolBar(KToggleAction* &act, const QString &name);
    void slotViewToolBarMain() { slotViewToolBar(actViewToolBarMain, "main"); };
    void slotViewToolBarActions() { slotViewToolBar(actViewToolBarActions, "actions"); };
    void slotViewStatusBar();

    void slotStatusMsg(const QString &text);
    void slotStatusHelpMsg(const QString& text);
    
    void slotLoadMode();
            
    void slotTransAvailable();
    void slotTransToTengwar();
    void slotTransToRoman();
    void slotRomanTextChanged();
    void slotTengwarTextChanged();

    void slotOptions();
   
    void slotWebHome();
    void slotWebYaTT();
    
  public:
    KConfig *config;
    KTTConfig configStruct;
    
    KTTView *view;
    KTTDoc *doc;

    // Narmacil transcription engine
    CTranscription *narmacil;

  protected:      
    // action pointers
    KAction* actFileNew;
    KAction* actFileOpenRoman;
    KAction* actFileOpenTengwar;
    KRecentFilesAction* actFileOpenRecent;
    KAction* actFileSave;
    KAction* actFileSaveAs;
    KAction* actFilePrint;
    KAction* actFileQuit;

    KAction* actEditCut;
    KAction* actEditCopy;
    KAction* actEditPaste;

    KToggleAction* actViewToolBarMain;
    KToggleAction* actViewToolBarActions;
    KToggleAction* actViewStatusBar;

    KAction* actTransToTengwar;
    KAction* actTransToRoman;
    KToggleAction* actAutoTranscribe;

    KAction* actOptions;
    KAction* actLoadMode;

    KAction* actWebHome;
    KAction* actWebYaTT;
};
 
#endif // KTT_H
