/***************************************************************************
   kttcodec.cpp  -  KTTCodec

   begin     : 2002.11.22
   copyright : (C) 2002 by MiMaS
   email     : mimas@mimas.ceti.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kttcodec.h"

#include <qstring.h>
#include <qcstring.h>
#include <qregexp.h>
#include <qtextcodec.h> 

#include <kmessagebox.h>

QString KTTCodec::toWindows(const QString inText, bool casesens)
{
  return encode( inText, casesens, true );
}

QString KTTCodec::fromWindows(const QString inText, bool casesens)
{
  return encode( inText, casesens, false );
}                                          

QString KTTCodec::encode(const QString inText, bool casesens, bool toWin)
{
  struct { char utf[3]; char win[2]; char winLo[2]; } codec[] =
  {
//  {"\xC3\xB3", "\xF3", "\xF3"}, //  o    /* polish */
    {"\xC4\x85", "\xB9", "\xB9"}, //  a
    {"\xC4\x87", "\xE6", "\xE6"}, //  c
    {"\xC4\x99", "\xEA", "\xEA"}, //  e
    {"\xC5\x82", "\xB3", "\xB3"}, //  l
    {"\xC5\x84", "\xF1", "\xF1"}, //  n
    {"\xC5\x9B", "\x9C", "\x9C"}, //  s
    {"\xC5\xBC", "\xBF", "\xBF"}, //  z
    {"\xC5\xBA", "\x9F", "\x9F"}, //  x

//  {"\xC3\x93", "\xD3", "\xF3"}, //  O
    {"\xC4\x84", "\xA5", "\xB9"}, //  A
    {"\xC4\x86", "\xC6", "\xE6"}, //  C
    {"\xC4\x98", "\xCA", "\xEA"}, //  E
    {"\xC5\x81", "\xA3", "\xB3"}, //  L
    {"\xC5\x83", "\xD1", "\xF1"}, //  N
    {"\xC5\x9A", "\x8C", "\x9C"}, //  S
    {"\xC5\xBB", "\xAF", "\xBF"}, //  Z
    {"\xC5\xB9", "\x8F", "\x9F"}, //  X

    {"\xC4\x8D", "\xE8", "\xE8"}, // c      /* czech */
    {"\xC4\x8F", "\xEF", "\xEF"}, // d
    {"\xC4\x9B", "\xEC", "\xEC"}, // e
    {"\xC5\x88", "\xF2", "\xF2"}, // n
    {"\xC5\x99", "\xF8", "\xF8"}, // r
    {"\xC5\xA1", "\x9A", "\x9A"}, //
    {"\xC5\xA5", "\x9D", "\x9D"}, //
    {"\xC5\xBE", "\x9E", "\x9E"}, // 
    {"\xC5\xAF", "\xF9", "\xF9"}, // u

    {"\xC4\x86", "\xC8", "\xE8"}, // C
    {"\xC4\x8E", "\xCF", "\xEF"}, // D
    {"\xC4\x9A", "\xCC", "\xEC"}, // E
    {"\xC5\x87", "\xD2", "\xF2"}, // N
    {"\xC5\x98", "\xD8", "\xF8"}, // R
    {"\xC5\xA0", "\x8A", "\x9A"}, //
    {"\xC5\xA4", "\x8D", "\x9D"}, //
    {"\xC5\xBD", "\x8E", "\x9E"}, //
    {"\xC5\xAE", "\xD9", "\xF9"}, // U


    {"\xC3\xA0", "\xE0", "\xE0"},
    {"\xC3\xA1", "\xE1", "\xE1"},
    {"\xC3\xA2", "\xE2", "\xE2"},
    {"\xC3\xA3", "\xE3", "\xE3"},
    {"\xC3\xA4", "\xE4", "\xE4"},
    {"\xC3\xA5", "\xE5", "\xE5"},
    {"\xC3\xA6", "\xE6", "\xE6"},
    {"\xC3\xA7", "\xE7", "\xE7"},
    {"\xC3\xA8", "\xE8", "\xE8"},
    {"\xC3\xA9", "\xE9", "\xE9"}, //  
    {"\xC3\xAA", "\xEA", "\xEA"},
    {"\xC3\xAB", "\xEB", "\xEB"},
    {"\xC3\xAC", "\xEC", "\xEC"},
    {"\xC3\xAD", "\xED", "\xED"}, //  
    {"\xC3\xAE", "\xEE", "\xEE"},
    {"\xC3\xAF", "\xEF", "\xEF"},
    {"\xC3\xB0", "\xF0", "\xF0"},
    {"\xC3\xB1", "\xF1", "\xF1"},
    {"\xC3\xB2", "\xF2", "\xF2"},
    {"\xC3\xB3", "\xF3", "\xF3"}, //  
    {"\xC3\xB4", "\xF4", "\xF4"},
    {"\xC3\xB5", "\xF5", "\xF5"},
    {"\xC3\xB6", "\xF6", "\xF6"},
    {"\xC3\xB8", "\xF8", "\xF8"},
    {"\xC3\xB9", "\xF9", "\xF9"},
    {"\xC3\xBA", "\xFA", "\xFA"}, // 
    {"\xC3\xBB", "\xFB", "\xEB"},
    {"\xC3\xBC", "\xFC", "\xFC"},
    {"\xC3\xBD", "\xFD", "\xFD"}, // 
    {"\xC3\xBE", "\xFE", "\xFE"},
    {"\xC3\xBF", "\xFF", "\xFF"},

    {"\xC3\x80", "\xC0", "\xE0"},
    {"\xC3\x81", "\xC1", "\xE1"}, // 
    {"\xC3\x82", "\xC2", "\xE2"},
    {"\xC3\x83", "\xC3", "\xE3"},
    {"\xC3\x84", "\xC4", "\xE4"},
    {"\xC3\x85", "\xC5", "\xE5"},
    {"\xC3\x86", "\xC6", "\xE6"},
    {"\xC3\x87", "\xC7", "\xE7"},
    {"\xC3\x88", "\xC8", "\xE8"},
    {"\xC3\x89", "\xC9", "\xE9"}, // 
    {"\xC3\x8A", "\xCA", "\xEA"},
    {"\xC3\x8B", "\xCB", "\xEB"},
    {"\xC3\x8C", "\xCC", "\xEC"},
    {"\xC3\x8D", "\xCD", "\xED"}, // 
    {"\xC3\x8E", "\xCE", "\xEE"},
    {"\xC3\x8F", "\xCF", "\xEF"},
    {"\xC3\x90", "\xD0", "\xF0"},
    {"\xC3\x91", "\xD1", "\xF1"},
    {"\xC3\x92", "\xD2", "\xF2"},
    {"\xC3\x93", "\xD3", "\xF3"}, // 
    {"\xC3\x94", "\xD4", "\xF4"},
    {"\xC3\x95", "\xD5", "\xF5"},
    {"\xC3\x96", "\xD6", "\xF6"},
    {"\xC3\x98", "\xD8", "\xF8"},
    {"\xC3\x99", "\xD9", "\xF9"},
    {"\xC3\x9A", "\xDA", "\xFA"}, // 
    {"\xC3\x9B", "\xDB", "\xFB"},
    {"\xC3\x9C", "\xDC", "\xFC"},
    {"\xC3\x9D", "\xDD", "\xFD"}, // 
    {"\xC3\x9E", "\xDE", "\xFE"},
    {"\xC3\x9F", "\xDF", "\xDF"}, // not FF
                    
    {"", "", ""}
  };

  QString outText = "";
  
  if ( toWin )
  {
    outText = inText.utf8();
    for ( int i = 0; codec[i].win[0]; i++ )
      outText = outText.replace( QRegExp(codec[i].utf), casesens ? codec[i].win : codec[i].winLo );

///KMessageBox::information(0, outText );
  }
  else
  {
///KMessageBox::information(0, inText);
    outText = inText;
    for ( uint j = 0; j < inText.length(); j++ )
      for ( uint i = 0; codec[i].win[0]; i++ )
        if ( outText[j] == codec[i].win[0] )
        {
          outText.replace( j, 1, codec[i].utf );
          j++;
        }
    outText = QTextCodec::codecForName("utf8")->toUnicode(outText);

///outText = QTextCodec::codecForName("CP1250")->toUnicode(inText);
///KMessageBox::information(0, outText);

  }    
  return outText;
}
