/***************************************************************************
   kttsetupdlg.cpp  -  KTT Options Dialog

   begin     : 2002.12.18
   copyright : (C) 2002 by MiMaS
   email     : mimas@mimas.ceti.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for KDE
#include <klocale.h>
#include <kiconloader.h>
#include <kurlrequester.h>

// include files for Qt
#include <qlayout.h>
#include <qtextedit.h>
#include <qtabwidget.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

// application specific includes
#include "kttsetupdlg.h"
#include "resources.h"
#include "ktt.h"
#include "kttview.h"
#include "narmacil/transcription.h"


KTTSetupDlg::KTTSetupDlg(KTTApp *parent) :
    KDialogBase(KDialogBase::IconList, i18n("Configure KTT"),
                KDialogBase::Ok | KDialogBase::Cancel | KDialogBase::Default,
                KDialogBase::Ok, (QWidget *)parent, "setupdialog", true, true)
{
  a = parent;   // parent of KTTSetupDlg is KTTApp (main application window)

  QVBox *page;

  // general options page
  page = addVBoxPage(i18n("General"), QString::null, BarIcon("configure", KIcon::SizeMedium ) );
  generalOptionsPage = new GeneralOptionsPage( (QWidget *)page );

  // transcription options page
  page = addVBoxPage(i18n("Transcription"), QString::null, BarIcon("gear", KIcon::SizeMedium ) );
  transOptionsPage = new TransOptionsPage( (QWidget *)page );

  /* 'auto transcribe' checkbox will have exactly the same 'What's this?' text
     as corresponding menu / toolbar button */
  if ( !(a->actAutoTranscribe->whatsThis().isEmpty()) )
    QWhatsThis::add( transOptionsPage->cbAutoTranscribe, a->actAutoTranscribe->whatsThis() );

  // fonts page
  page = addVBoxPage(i18n("Fonts"), QString::null, BarIcon("fonts", KIcon::SizeMedium ) );

  /* there's no KFontChooser in Qt Designer,
    so fonts page widget is made here, not in options*.ui file */
  fontOptionsPage = new QWidget((QWidget *)page);
  fontOptionsPage->resize(509, 409);
  fontOptionsPage->setSizePolicy(QSizePolicy((QSizePolicy::SizeType)7, (QSizePolicy::SizeType)7, 0, 0, sizePolicy().hasHeightForWidth()));
  QGridLayout *fontOptionsPageLayout = new QGridLayout(fontOptionsPage, 1, 1, 11, 6);

  QTabWidget *tabWidget = new QTabWidget(fontOptionsPage);
  tabWidget->setSizePolicy(QSizePolicy((QSizePolicy::SizeType)7, (QSizePolicy::SizeType)7, 0, 0, tabWidget->sizePolicy().hasHeightForWidth()));

  QWidget *tabTengwar = new QWidget(tabWidget);
  QGridLayout *tabTengwarLayout = new QGridLayout(tabTengwar, 1, 1, 11, 6);
  fontChooserTengwar = new KFontChooser(tabTengwar, 0, false, a->configStruct.tengwarFonts, false);
  fontChooserTengwar->setSampleText("`VjR5 8~BjE jUt$5\" `Nt$4%`VjyY");
  // sample text is "Elen sila lumenn omentielvo" in Quenya using Dan Smith's keyboard layout
  tabTengwarLayout->addWidget(fontChooserTengwar, 0, 0);
  tabWidget->insertTab(tabTengwar, i18n("Tengwar Font"));

  QWidget *tabRoman = new QWidget(tabWidget);
  QGridLayout *tabRomanLayout = new QGridLayout(tabRoman, 1, 1, 11, 6);
  fontChooserRoman = new KFontChooser(tabRoman, 0, false, QStringList(), false);
  fontChooserRoman->setSampleText("Elen sila lumenn omentielvo");
  tabRomanLayout->addWidget(fontChooserRoman, 0, 0);
  tabWidget->insertTab(tabRoman, i18n("Roman Font"));

  fontOptionsPageLayout->addWidget(tabWidget, 0, 0);


  connect(this, SIGNAL(defaultClicked()), this, SLOT(slotDefault()));
}

KTTSetupDlg::~KTTSetupDlg()
{
}

void KTTSetupDlg::setOptions()
{
  fontChooserTengwar->setFont(a->view->getFontTengwar());
  fontChooserRoman->setFont(a->view->getFontRoman());

  transOptionsPage->cbAutoTranscribe->setChecked(a->configStruct.autoTranscribe);
  transOptionsPage->cbClearTarget->setChecked(a->configStruct.clearTarget);

  if ( a->configStruct.narmacil.alert )
    transOptionsPage->rbErrorsAlert->setChecked(true);
  else
    transOptionsPage->rbErrorsCont->setChecked(true);
  if ( a->configStruct.narmacil.decimal )
    transOptionsPage->rbBaseDecimal->setChecked(true);
  else
    transOptionsPage->rbBaseDuodecimal->setChecked(true);
  transOptionsPage->cbSmart->setChecked(a->configStruct.narmacil.smart);
  transOptionsPage->cbMarkDigits->setChecked(a->configStruct.narmacil.digits);
  transOptionsPage->cbMarkLSD->setChecked(a->configStruct.narmacil.LSD);
  transOptionsPage->cbSeparateNumbers->setChecked(a->configStruct.narmacil.separateNums);

  generalOptionsPage->rbStartupNone->setChecked(a->configStruct.onStartup == STARTUP_NONE);
  generalOptionsPage->rbStartupReload->setChecked(a->configStruct.onStartup == STARTUP_RELOAD);
  generalOptionsPage->rbStartupLoad->setChecked(a->configStruct.onStartup == STARTUP_LOAD);
  generalOptionsPage->urlRequester->setURL(a->configStruct.startupMode);
  generalOptionsPage->cbFullCaption->setChecked(a->configStruct.fullPathCaption);
}

void KTTSetupDlg::getOptions()
{
  a->configStruct.autoTranscribe = transOptionsPage->cbAutoTranscribe->isChecked();
  a->actAutoTranscribe->setChecked(transOptionsPage->cbAutoTranscribe->isChecked());
  a->configStruct.clearTarget = transOptionsPage->cbClearTarget->isChecked();

  a->configStruct.narmacil.alert = transOptionsPage->rbErrorsAlert->isChecked();
  a->configStruct.narmacil.smart = transOptionsPage->cbSmart->isChecked();
  a->configStruct.narmacil.digits = transOptionsPage->cbMarkDigits->isChecked();
  a->configStruct.narmacil.LSD = transOptionsPage->cbMarkLSD->isChecked();
  a->configStruct.narmacil.decimal = transOptionsPage->rbBaseDecimal->isChecked();
  a->configStruct.narmacil.separateNums = transOptionsPage->cbSeparateNumbers->isChecked();

  if ( a->narmacil )
  {
    a->narmacil->SetAlert(a->configStruct.narmacil.alert);
    a->narmacil->SetSmart(a->configStruct.narmacil.smart);
    a->narmacil->SetDigits(a->configStruct.narmacil.digits);
    a->narmacil->SetLSD(a->configStruct.narmacil.LSD);
    a->narmacil->SetDecimal(a->configStruct.narmacil.decimal);
    a->narmacil->SetSeparateNums(a->configStruct.narmacil.separateNums);
  }
  
  
  if (generalOptionsPage->rbStartupReload->isChecked())
    a->configStruct.onStartup = STARTUP_RELOAD;
  else if (generalOptionsPage->rbStartupLoad->isChecked())
    a->configStruct.onStartup = STARTUP_LOAD;
  else
    a->configStruct.onStartup = STARTUP_NONE;
  a->configStruct.startupMode = generalOptionsPage->urlRequester->url();
  a->configStruct.fullPathCaption = generalOptionsPage->cbFullCaption->isChecked();

  a->view->setFontTengwar(fontChooserTengwar->font());
  a->view->setFontRoman(fontChooserRoman->font());
}

void KTTSetupDlg::slotDefault()
{
//  fontChooserTengwar->setFont(a->view->getFontTengwar());
//  fontChooserRoman->setFont(a->view->getFontRoman());

  if ( transOptionsPage->isVisible() )
  {
    transOptionsPage->cbAutoTranscribe->setChecked(false);
    transOptionsPage->cbClearTarget->setChecked(true);
    transOptionsPage->rbErrorsAlert->setChecked(false);
    transOptionsPage->rbErrorsCont->setChecked(true);
    transOptionsPage->rbBaseDecimal->setChecked(false);
    transOptionsPage->rbBaseDuodecimal->setChecked(true);
    transOptionsPage->cbSmart->setChecked(true);
    transOptionsPage->cbMarkDigits->setChecked(false);
    transOptionsPage->cbMarkLSD->setChecked(false);
    transOptionsPage->cbSeparateNumbers->setChecked(true);
  }

  if ( generalOptionsPage->isVisible() )
  {
    generalOptionsPage->rbStartupNone->setChecked(true);
    generalOptionsPage->rbStartupReload->setChecked(false);
    generalOptionsPage->rbStartupLoad->setChecked(false);
    generalOptionsPage->urlRequester->setURL("");
    generalOptionsPage->cbFullCaption->setChecked(false);
  }
}


