/***************************************************************************
   kttview.cpp  -  KTTView

   begin     : 2002.11.08
   copyright : (C) 2002 by MiMaS
   email     : mimas@mimas.ceti.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qprinter.h>
#include <qpainter.h>
#include <qtextedit.h>
#include <qvariant.h>
#include <qpushbutton.h>
#include <qsplitter.h>
#include <qtextedit.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

// include files for KDE
#include <kled.h>

// application specific includes
#include "kttview.h"
#include "kttdoc.h"
#include "ktt.h"

KTTView::KTTView(QWidget *parent) : QWidget(parent, "MainForm")
{
// below is the code as generated by Qt Designer
//      mainform.ui
//      $ uic -o mainform.h mainform.ui
//      $ uic -o mainform.cpp -impl mainform.h mainform.ui
// the only change is sigQTextEdit in place of QTextEdit
// <QtDesigner>
  setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)7, 0, 0, sizePolicy().hasHeightForWidth() ) );
  MainFormLayout = new QGridLayout( this, 1, 1, 0, 6, "MainFormLayout");

  Splitter = new QSplitter( this, "Splitter" );
  Splitter->setOrientation( QSplitter::Horizontal );

  QWidget* privateLayoutWidget = new QWidget( Splitter, "LayoutLeft" );
  LayoutLeft = new QVBoxLayout( privateLayoutWidget, 0, 1, "LayoutLeft");

  TextEditLeft = new sigQTextEdit( privateLayoutWidget, "TextEditLeft" );
  TextEditLeft->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)7, 0, 0, TextEditLeft->sizePolicy().hasHeightForWidth() ) );
  TextEditLeft->setMargin( 0 );
  TextEditLeft->setTextFormat( QTextEdit::PlainText );
  LayoutLeft->addWidget( TextEditLeft );

  LayoutLeftDown = new QHBoxLayout( 0, 3, 6, "LayoutLeftDown");
  
  LedLeft = new KLed( privateLayoutWidget, "LedLeft" );
  LedLeft->setState( KLed::Off );
  LedLeft->setLook( KLed::Sunken );
  LedLeft->setColor( QColor( 0, 255, 0 ) );
  LedLeft->setDarkFactor( 350 );
  LayoutLeftDown->addWidget( LedLeft );

  LedModLeft = new KLed( privateLayoutWidget, "LedModLeft" );
  LedModLeft->setState( KLed::Off );
  LedModLeft->setLook( KLed::Sunken );
  LedModLeft->setColor( QColor( 255, 255, 0 ) );
  LedModLeft->setDarkFactor( 350 );
  QToolTip::add( LedModLeft, i18n("modified") );
  LayoutLeftDown->addWidget( LedModLeft );

  QSpacerItem* spacer = new QSpacerItem( 30, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
  LayoutLeftDown->addItem( spacer );
  LayoutLeft->addLayout( LayoutLeftDown );
  
  QWidget* privateLayoutWidget_2 = new QWidget( Splitter, "LayoutRight" );
  LayoutRight = new QVBoxLayout( privateLayoutWidget_2, 0, 1, "LayoutRight");

  TextEditRight = new sigQTextEdit( privateLayoutWidget_2, "TextEditRight" );
  TextEditRight->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)7, 0, 0, TextEditRight->sizePolicy().hasHeightForWidth() ) );
  QFont TextEditRight_font(  TextEditRight->font() );
  TextEditRight_font.setFamily( "Tengwar Quenya" );
  TextEditRight->setFont( TextEditRight_font );
  TextEditRight->setTextFormat( QTextEdit::PlainText );
  LayoutRight->addWidget( TextEditRight );

  LayoutRightDown = new QHBoxLayout( 0, 3, 6, "LayoutRightDown");
  
  LedRight = new KLed( privateLayoutWidget_2, "LedRight" );
  LedRight->setState( KLed::Off );
  LedRight->setLook( KLed::Sunken );
  LedRight->setColor( QColor( 0, 255, 0 ) );
  LedRight->setDarkFactor( 350 );
  LayoutRightDown->addWidget( LedRight );

  LedModRight = new KLed( privateLayoutWidget_2, "LedModRight" );
  LedModRight->setState( KLed::Off );
  LedModRight->setLook( KLed::Sunken );
  LedModRight->setColor( QColor( 255, 255, 0 ) );  
  LedModRight->setDarkFactor( 350 );
  QToolTip::add( LedModRight, i18n("modified") );
  LayoutRightDown->addWidget( LedModRight );

  QSpacerItem* spacer_2 = new QSpacerItem( 30, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
  LayoutRightDown->addItem( spacer_2 );
  LayoutRight->addLayout( LayoutRightDown );
  
  MainFormLayout->addWidget( Splitter, 0, 0 );
// </QtDesigner>


  QWhatsThis::add( LedLeft, i18n("When this LED is on, <b>Roman</b> pane is active, "
    "and <i>Save</i>, <i>Print</i> and <i>editing</i> actions works on Roman text.") );
  QWhatsThis::add( LedRight, i18n("When this LED is on, <b>Tengwar</b> pane is active, "
    "and <i>Save</i>, <i>Print</i> and <i>editing</i> actions works on Tengwar text.") );
  // compare other "What's This" help texts in ktt_whatsthis.cpp
    
  setBackgroundMode(PaletteBase);

  connect(TextEditLeft, SIGNAL(copyAvailable(bool)), parent, SLOT(slotCopyAvailable(bool)));
  connect(TextEditLeft, SIGNAL(textChanged()), parent, SLOT(slotRomanTextChanged()));
  connect(TextEditLeft, SIGNAL(focusChange()), this, SLOT(slotTextEditFocusChanged()));
  connect(TextEditLeft, SIGNAL(modificationChanged(bool)), this, SLOT(slotTextEditModificationChanged(bool)));
    
  connect(TextEditRight, SIGNAL(copyAvailable(bool)), parent, SLOT(slotCopyAvailable(bool)));
  connect(TextEditRight, SIGNAL(textChanged()), parent, SLOT(slotTengwarTextChanged()));
  connect(TextEditRight, SIGNAL(focusChange()), this, SLOT(slotTextEditFocusChanged()));
  connect(TextEditRight, SIGNAL(modificationChanged(bool)), this, SLOT(slotTextEditModificationChanged(bool)));
}

KTTView::~KTTView()
{
}

KTTDoc *KTTView::getDocument() const
{
  KTTApp *theApp=(KTTApp *) parentWidget();
  return theApp->getDocument();
}

void KTTView::print(QPrinter *pPrinter)
{
  QPainter printpainter;
  printpainter.begin(pPrinter);

  switch ( currentDocumentType )
  {
    case DOCUMENT_TYPE_ROMAN:
      printpainter.setFont(fontRoman);
      printpainter.drawText(rect(), AlignLeft, textRoman(true));
      break;
    case DOCUMENT_TYPE_TENGWAR:
      printpainter.setFont(fontTengwar);
      printpainter.drawText(rect(), AlignLeft, textTengwar(true));
      break;
  }

  printpainter.end();
}

void KTTView::slotTextEditFocusChanged()
{
  if ( TextEditLeft->hasFocus() )
  {
    LedRight->off();
    LedLeft->on();
    currentDocumentType = DOCUMENT_TYPE_ROMAN;
  }
  if ( TextEditRight->hasFocus() )
  {
    LedLeft->off();
    LedRight->on();
    currentDocumentType = DOCUMENT_TYPE_TENGWAR;
  }
  // if neither TextEditLeft nor TextEditRight hasFocus()
  // then leds are not changed
}

void KTTView::slotTextEditModificationChanged(bool)
{
  if ( TextEditLeft->isModified() )
    LedModLeft->on();
  else
    LedModLeft->off();

  if ( TextEditRight->isModified() )
    LedModRight->on();
  else
    LedModRight->off();
}

void KTTView::cut()
{
  switch ( currentDocumentType )
  {
    case DOCUMENT_TYPE_ROMAN:
      TextEditLeft->cut();
      break;
    case DOCUMENT_TYPE_TENGWAR:
      TextEditRight->cut();
      break;
  }
}

void KTTView::copy()
{
  switch ( currentDocumentType )
  {
    case DOCUMENT_TYPE_ROMAN:
      TextEditLeft->copy();
      break;
    case DOCUMENT_TYPE_TENGWAR:
      TextEditRight->copy();
      break;
  }
}

void KTTView::paste()
{
  switch ( currentDocumentType )
  {
    case DOCUMENT_TYPE_ROMAN:
      TextEditLeft->paste();
      break;
    case DOCUMENT_TYPE_TENGWAR:
      TextEditRight->paste();
      break;
  }
}

void KTTView::clearTengwar()
{
  TextEditRight->clear();
}

void KTTView::clearRoman()
{
  TextEditLeft->clear();
}

void KTTView::rewindBoth()
{
  TextEditLeft->setCursorPosition(0,0);
  TextEditRight->setCursorPosition(0,0);
}

QString KTTView::textTengwar(bool selectedOnly) const
{
  QString t;
  if ( selectedOnly && TextEditRight->hasSelectedText() )
   t = TextEditRight->selectedText();
  else
   t = TextEditRight->text();
  return t;
};

QString KTTView::textRoman(bool selectedOnly) const
{
  QString t;
  if ( selectedOnly && TextEditLeft->hasSelectedText() )
   t = TextEditLeft->selectedText();
  else
   t = TextEditLeft->text();
  return t;
};

void KTTView::setTengwar(const QString & text, bool clear)
{
  if (clear)
    TextEditRight->setText(text);
  else
    TextEditRight->append(text);
  TextEditRight->selectAll(true);
  TextEditRight->setCurrentFont(fontTengwar);
  TextEditRight->selectAll(false);
}

void KTTView::setRoman(const QString & text, bool clear)
{
  if (clear)
    TextEditLeft->setText(text);
  else
    TextEditLeft->append(text);
  TextEditLeft->selectAll(true);
  TextEditLeft->setCurrentFont(fontRoman);
  TextEditLeft->selectAll(false);
}

void KTTView::setFontTengwar(const QFont & font)
{
  fontTengwar = font;
  TextEditRight->setCurrentFont(fontTengwar);
}

void KTTView::setFontRoman(const QFont & font)
{
  fontRoman = font;
  TextEditLeft->setCurrentFont(fontRoman);
}  


